package com.isu.gaswellwatch.modbus.data.decode.impl;

import com.isu.gaswellwatch.modbus.data.ModbusMessage;
import com.isu.gaswellwatch.modbus.data.decode.DecodeHandler;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.math.NumberUtils;
import org.springframework.stereotype.Component;

import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * 本地时间类型解析器
 *
 * @author <a href="mailto:scwsl@foxmail.com">王仕龙</a>
 * 2024/12/09 14:40
 */
@Component(SecondLocalTimeDecodeHandler.NAME + DecodeHandler.DECODE_NAME)
public class SecondLocalTimeDecodeHandler implements DecodeHandler {
    public static final String NAME = "secondLocalTime";

    @Override
    public String decode(Map<String, Object> commandPointMap, String value) {
        if (StringUtils.isNotBlank(value) && NumberUtils.isCreatable(value)) {
            long totalSeconds = NumberUtils.createLong(value);
            long hours = TimeUnit.SECONDS.toHours(totalSeconds);

            long remainingSeconds = totalSeconds - TimeUnit.HOURS.toSeconds(hours);
            long minutes = TimeUnit.SECONDS.toMinutes(remainingSeconds);

            remainingSeconds = remainingSeconds - TimeUnit.MINUTES.toSeconds(minutes);

            return String.format("%02d:%02d:%02d", hours, minutes, remainingSeconds);
        }
        return value;
    }

    @Override
    public void decode(Map<String, Object> commandPointMap, ModbusMessage.MessagePoint point) {
        point.setValue(this.decode(commandPointMap, point.getValue()));
    }

}
