package com.isu.gaswellwatch.vo.command.scss;

import com.isu.gaswellwatch.dto.modbus.ModbusCommandDto;
import com.isu.gaswellwatch.vo.command.Command;
import jakarta.validation.constraints.NotNull;
import lombok.Getter;
import lombok.Setter;
import lombok.experimental.SuperBuilder;
import org.apache.commons.lang3.StringUtils;

import java.io.Serial;
import java.time.LocalDateTime;
import java.time.temporal.ChronoField;
import java.util.Collection;
import java.util.List;
import java.util.Objects;

/**
 * @author <a href="mailto:scwsl@foxmail.com">王仕龙</a>
 * 2024/11/27 23:53
 */
@Getter
@Setter
@SuperBuilder
public class SystemInfo extends Command {

    @Serial
    private static final long serialVersionUID = -4927792179148968016L;

    /**
     * 日期时间, yyyy-MM-dd HH:mm:ss.SSS
     */
    @NotNull(message = "系统时间不能为空")
    private LocalDateTime dataTime;
    /**
     * 显示延时
     */
    private int showDelay = 0;
    /**
     * 开井采样间隔
     */
    private int openWellSamplingInterval = 0;
    /**
     * 关井采样间隔
     */
    private int closeWellSamplingInterval = 0;

    public SystemInfo() {
        this.setCode("SCSS.SYSTEM_INFO");
    }

    @Override
    public boolean validate() {
        return super.validate() && Objects.isNull(this.dataTime);
    }

    @Override
    protected Collection<ModbusCommandDto> builderModbusCommand() {
        StringBuilder command = new StringBuilder(100);
        //                                0014*2->hex
        // 地址码  功能码  起始地址  连续长度  连续字长
        // 01     10     0000    0014     28
        command.append("01100000001428");
        command.append(StringUtils.leftPad(Integer.toHexString(this.dataTime.getYear() - 2000), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.dataTime.getMonthValue()), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.dataTime.getDayOfMonth()), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.dataTime.getHour()), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.dataTime.getMinute()), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.dataTime.getSecond()), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.dataTime.get(ChronoField.MILLI_OF_SECOND)), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.showDelay), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.openWellSamplingInterval), 8, "0"));
        command.append(StringUtils.leftPad(Integer.toHexString(this.closeWellSamplingInterval), 8, "0"));
        return List.of(ModbusCommandDto.builder().command(command.toString()).length(16).build());
    }

}
